@@ -68,7 +68,7 @@ mousepad_dialogs_show_about (GtkWindow *parent)
   gtk_show_about_dialog (parent,
                          "authors", authors,
                          "comments", _("Mousepad is a simple text editor for the Xfce desktop environment"),
-                         "copyright", "Copyright \xc2\xa9 2005-2023 - the Mousepad developers",
+                         "copyright", "Copyright \xc2\xa9 2005-2024 - the Mousepad developers",
                          "destroy-with-parent", TRUE,
                          "license-type", GTK_LICENSE_GPL_2_0,
                          "logo-icon-name", MOUSEPAD_ID,
@@ -639,6 +639,7 @@ mousepad_dialogs_session_restore (void)
                                    GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                                    _("It seems that the previous session did not end normally."
                                      " Do you want to restore the available data?"));
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
   /* destroy with parent or on "app.quit" */
   if (parent != NULL)
@@ -872,7 +873,15 @@ mousepad_dialogs_combo_changed (GtkComboBox *combo,
           /* try to create a temporary file to save the current buffer */
           if ((g_file = g_file_new_tmp (NULL, &iostream, &error)) != NULL)
             {
-              file = mousepad_file_new (mousepad_file_get_buffer (current_file));
+              GtkTextBuffer *buffer = mousepad_file_get_buffer (current_file);
+              GtkTextIter start, end;
+              gchar *text;
+              gtk_text_buffer_get_start_iter (buffer, &start);
+              gtk_text_buffer_get_end_iter (buffer, &end);
+              text = gtk_text_buffer_get_text (buffer, &start, &end, TRUE);
+              buffer = GTK_TEXT_BUFFER (gtk_source_buffer_new (NULL));
+              gtk_text_buffer_set_text (buffer, text, -1);
+              file = mousepad_file_new (buffer);
               mousepad_file_set_location (file, g_file, MOUSEPAD_LOCATION_REAL);
               if (encoding != MOUSEPAD_ENCODING_NONE)
                 mousepad_file_set_encoding (file, encoding);
@@ -882,6 +891,8 @@ mousepad_dialogs_combo_changed (GtkComboBox *combo,
               mousepad_file_save (file, FALSE, &error);
 
               /* cleanup */
+              g_free (text);
+              g_object_unref (buffer);
               g_object_unref (g_file);
               g_object_unref (iostream);
             }
