@@ -26,6 +26,8 @@
 
 
 static void      mousepad_document_finalize                (GObject                *object);
+static gboolean  mousepad_document_scroll_event            (GtkWidget              *widget,
+                                                            GdkEventScroll         *event);
 static void      mousepad_document_notify_cursor_position  (MousepadDocument       *document);
 static void      mousepad_document_encoding_changed        (MousepadFile           *file,
                                                             MousepadEncoding        encoding,
@@ -111,10 +113,14 @@ static void
 mousepad_document_class_init (MousepadDocumentClass *klass)
 {
   GObjectClass *gobject_class;
+  GtkWidgetClass *gtkwidget_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = mousepad_document_finalize;
 
+  gtkwidget_class = GTK_WIDGET_CLASS (klass);
+  gtkwidget_class->scroll_event = mousepad_document_scroll_event;
+
   document_signals[CLOSE_TAB] =
     g_signal_new (I_("close-tab"), G_TYPE_FROM_CLASS (gobject_class), G_SIGNAL_RUN_LAST,
                   0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
@@ -305,6 +311,21 @@ mousepad_document_finalize (GObject *object)
 
 
 
+static gboolean
+mousepad_document_scroll_event (GtkWidget      *widget,
+                                GdkEventScroll *event)
+{
+  g_return_val_if_fail (MOUSEPAD_IS_DOCUMENT (widget), FALSE);
+
+  /* do not scroll text while zooming in or out */
+  if (event->state & GDK_CONTROL_MASK)
+    return TRUE;
+
+  return GTK_WIDGET_CLASS (mousepad_document_parent_class)->scroll_event (widget, event);
+}
+
+
+
 static void
 mousepad_document_notify_cursor_position (MousepadDocument *document)
 {
@@ -680,6 +701,9 @@ mousepad_document_search_completed_idle (gpointer data)
   else if (! (flags & MOUSEPAD_SEARCH_FLAGS_ACTION_NONE)
            && ! (flags & MOUSEPAD_SEARCH_FLAGS_AREA_SELECTION))
     gtk_text_buffer_place_cursor (document->buffer, &iter);
+  /* if e.g. a silent search fails by modifying the search entry this must be reset */
+  else
+    document->priv->cur_match = 0;
 
   /* force the signal emission, to cover cases where Mousepad search settings change without
    * changing GtkSourceView settings (e.g. when switching between single-document mode and
