@@ -38,6 +38,7 @@ static void     mousepad_history_autosave_init              (void);
 static void     mousepad_history_autosave_finalize          (void);
 static void     mousepad_history_search_init                (void);
 static void     mousepad_history_search_finalize            (void);
+static void     mousepad_history_paste_finalize             (void);
 
 
 
@@ -78,6 +79,11 @@ static GList *autosave_ids = NULL;
 static GHashTable *search_history = NULL;
 static GHashTable *replace_history = NULL;
 
+/* paste data */
+#define PASTE_HISTORY_MENU_WIDTH  30
+#define PASTE_HISTORY_MENU_LENGTH 10
+static GSList *clipboard_history = NULL;
+
 
 
 void
@@ -96,6 +102,7 @@ mousepad_history_finalize (void)
 {
   mousepad_history_autosave_finalize ();
   mousepad_history_search_finalize ();
+  mousepad_history_paste_finalize ();
 }
 
 
@@ -1205,3 +1212,128 @@ mousepad_history_search_insert_replace_text (const gchar *text)
   return mousepad_history_search_insert_text (text, replace_history,
                                               MOUSEPAD_SETTING_SEARCH_REPLACE_HISTORY);
 }
+
+
+
+static void
+mousepad_history_paste_finalize (void)
+{
+  g_slist_free_full (clipboard_history, g_free);
+}
+
+
+
+void
+mousepad_history_paste_add (void)
+{
+  GSList *li;
+  gchar *text;
+  gint n;
+
+  /* get the current clipboard text */
+  text = gtk_clipboard_wait_for_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD));
+
+  /* leave when there is no text */
+  if (G_UNLIKELY (text == NULL || *text == '\0'))
+    return;
+
+  clipboard_history = g_slist_prepend (clipboard_history, text);
+
+  /* remove eventual duplicate */
+  for (li = clipboard_history->next, n = 1; li != NULL && n < PASTE_HISTORY_MENU_LENGTH; n++)
+    {
+      GSList *lnext = li->next;
+      if (g_strcmp0 (li->data, text) == 0)
+        {
+          g_free (li->data);
+          clipboard_history = g_slist_delete_link (clipboard_history, li);
+        }
+      li = lnext;
+    }
+
+  /* truncate list if needed */
+  if (li != NULL)
+    {
+      g_free (li->data);
+      clipboard_history = g_slist_delete_link (clipboard_history, li);
+    }
+}
+
+
+
+static GtkWidget *
+get_menu_item (const gchar *text)
+{
+  GtkWidget *item;
+  GString *string;
+  gchar *label_str;
+
+  /* create new label string */
+  string = g_string_sized_new (PASTE_HISTORY_MENU_WIDTH);
+
+  /* get the first 30 chars of the clipboard text */
+  if (g_utf8_strlen (text, -1) > PASTE_HISTORY_MENU_WIDTH)
+    {
+      /* append the first 30 chars */
+      const gchar *s = g_utf8_offset_to_pointer (text, PASTE_HISTORY_MENU_WIDTH);
+      string = g_string_append_len (string, text, s - text);
+
+      /* make it look like a ellipsized string */
+      string = g_string_append (string, "...");
+    }
+  else
+    {
+      /* append the entire string */
+      string = g_string_append (string, text);
+    }
+
+  /* get the string */
+  label_str = g_string_free (string, FALSE);
+
+  /* replace tab and new lines with spaces */
+  label_str = g_strdelimit (label_str, "\n\t\r", ' ');
+
+  /* create a new item */
+  item = gtk_menu_item_new_with_label (label_str);
+
+  /* cleanup */
+  g_free (label_str);
+
+  return item;
+}
+
+
+
+GtkWidget *
+mousepad_history_paste_get_menu (GCallback callback,
+                                 gpointer data)
+{
+  GtkWidget *menu, *item;
+
+  /* create new menu and set the screen */
+  menu = gtk_menu_new ();
+  g_object_ref_sink (menu);
+  g_signal_connect (menu, "deactivate", G_CALLBACK (g_object_unref), NULL);
+
+  /* append the history items */
+  for (GSList *li = clipboard_history; li != NULL; li = li->next)
+    {
+      /* create menu item */
+      item = get_menu_item (li->data);
+      mousepad_object_set_data (item, "history-pointer", li->data);
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+      g_signal_connect (item, "activate", callback, data);
+      gtk_widget_show (item);
+    }
+
+  /* create an item to inform the user if history is empty */
+  if (!mousepad_util_container_has_children (GTK_CONTAINER (menu)))
+    {
+      item = gtk_menu_item_new_with_label (_("No clipboard data"));
+      gtk_widget_set_sensitive (item, FALSE);
+      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+      gtk_widget_show (item);
+    }
+
+  return menu;
+}
